
/* Router solicitation program.  Standalone.
   Does not require link-layer indications */


/*************************************************************/
/*
Please note that either this license MUST be displayed
or a particular instance of this code may be converted to
the GNU General Public License v2 or greater.

Please see README.txt for details.
*/

/*
Copyright (c) 2005, Monash University
All rights reserved.
Contributors: Greg Daley

Redistribution and use in source and binary forms, with or
without modification, are permitted provided that the following
conditions are met:

* Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above
  copyright notice, this list of conditions and the following
  disclaimer in the documentation and/or other materials
  provided with the distribution.

* Neither the name of Monash University nor the names of its
  contributors may be used to endorse or promote products derived
  from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
 
*/
/*************************************************************/

/*  GD: Note to self: 20040918
    GD: Note to self: 20050110 evtest

     REF RFC 3549, for RTNETLINK msg  formats and types.


*/


/**************************************************************************/

/*ipv6 socket headers  AAA */
#include <sys/socket.h>
#include <netinet/in.h>
/* interface request headers  BBB */
#include <sys/ioctl.h>
#include <net/if.h>
/* extra ifreq headers*/
#include <string.h>

/* rtnetlink socket includes */
#include <asm/types.h>
#include <linux/netlink.h>
#include <linux/rtnetlink.h>
#include <sys/socket.h>


/* local diagnostic headers */
#include <stdio.h>
#include <stdlib.h>		/* exit(1)... */
#include <errno.h>

/* time information */
#include <time.h>
#include <sys/time.h>



extern int send_rs(int sock, int ifindex, uint8_t macaddr[], int tentative,
			struct in6_addr *src, struct in6_addr *dst, uint8_t *
			prefix);


extern int open_v6sock(void);
extern int get_ifindex(int sock, const char *name);
extern int bind_rtsock(uint32_t groups);
extern int get_mac_rtnetlink(int sock, int ifindex, unsigned char *);
extern int get_linklocal_rtnetlink(int sock, int ifindex, unsigned char *);

/**************************************************************************/


int main(int argc, char *argv[])
{
	int v6sock;
	/*char ifname[IFNAMSIZ]="eth1"; */
	char name[IFNAMSIZ + 1];

	int ifindex;
	int rtnetlink_socket;
	uint8_t macaddr[6];
	uint8_t prefix[8];
	int macaddrset = 0;
	struct in6_addr *src = NULL;
	struct in6_addr srcbuff;
	int usesrcaddr = 0;
	int tentative = 0;

    /***********************************************************************/

	memset(macaddr, 0, 6 * sizeof(uint8_t));
	memset(prefix, 0, 8 * sizeof(uint8_t));

	if ((argc > 2) && (argv[argc-1][0] != '-')&& (argv[argc-2][0] == '-')
			&& (argv[argc-2][1] == 'L')) {
		sscanf(argv[argc - 1],
			"%02hhx%02hhx:%02hhx%02hhx:%02hhx%02hhx:%02hhx%02hhx",
			&prefix[0], &prefix[1], &prefix[2], &prefix[3],
			&prefix[4], &prefix[5], &prefix[6], &prefix[7]);
		argc-=2;
        }
	if ((argc > 2) && (argv[argc - 1][0] != '-')) {
		sscanf(argv[argc - 1],
		       "%02hhx:%02hhx:%02hhx:%02hhx:%02hhx:%02hhx",
		       &macaddr[0], &macaddr[1], &macaddr[2], &macaddr[3],
		       &macaddr[4], &macaddr[5]);
		macaddrset = 1;
		argc--;
	}
	if ((argc == 4) && !strncmp("-t", argv[3], 3)) {
		tentative = 1;
		argc = 3;
	}

	if ((argc == 3) && !strncmp("-s", argv[2], 3)) {
		usesrcaddr = 1;
		argc = 2;
	}
	if (argc != 2) {
		fprintf(stderr,
			"Usage: %s <interface> [-s  [-t] [<ethmac>] [-L <prefix>]\n"
			"\t-s:  Use unicast source address\n"
			"\t-t:  Use tentative link-layer addresses\n"
			"\t<ethmac>:  XX:XX:XX:XX:XX:XX\n"
			"\t<prefix>:  XXXX:XXXX:XXXX:XXXX\n", argv[0]);
		exit(1);
	}
	strncpy(name, argv[1], IFNAMSIZ);




    /***********************************************************************/


	if ((v6sock = open_v6sock()) < 0) {
		fprintf(stderr, "unable to get ipv6 socket\n");
		exit(1);
	}

	if( (rtnetlink_socket = bind_rtsock(0)) < 0 ){
		fprintf(stderr, "unable to get netlink socket\n");
		exit(1);
	}


	if ((ifindex = get_ifindex(v6sock, name)) < 0) {
		fprintf(stderr, "unable to get ifindex\n");
		exit(1);
	}

	if (!macaddrset && get_mac_rtnetlink(rtnetlink_socket, ifindex, 
			(unsigned char *)macaddr) < 0) {
		printf("mac address setup failed on %s\n", name);
		return -1;
	}
	if (usesrcaddr) {
		src = &srcbuff;
		if (get_linklocal_rtnetlink(rtnetlink_socket, ifindex,
			(unsigned char *) src) < 0) {
			printf("lladdr setup failed %s\n", name);
			return -1;
		}
	}

    /***********************************************************************/

	send_rs(v6sock, ifindex, macaddr, tentative, src,
						 NULL, prefix);
	return 0;
}


